package cn.com.infosec.crypto.generators;

import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.DerivationFunction;
import cn.com.infosec.crypto.DerivationParameters;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.params.ISO18033KDFParameters;
import cn.com.infosec.crypto.params.KDFParameters;

public class BaseKDFBytesGenerator implements DerivationFunction {
   private int counterStart;
   private Digest digest;
   private byte[] shared;
   private byte[] iv;

   protected BaseKDFBytesGenerator(int counterStart, Digest digest) {
      this.counterStart = counterStart;
      this.digest = digest;
   }

   public void init(DerivationParameters param) {
      if (param instanceof KDFParameters) {
         BaseKDFBytesGenerator var10000 = this;
         KDFParameters var2;
         this.shared = (var2 = (KDFParameters)param).getSharedSecret();
         var10000.iv = var2.getIV();
      } else {
         if (!(param instanceof ISO18033KDFParameters)) {
            throw new IllegalArgumentException("KDF parameters required for KDF2Generator");
         }

         this.shared = ((ISO18033KDFParameters)param).getSeed();
         this.iv = null;
      }

   }

   public Digest getDigest() {
      return this.digest;
   }

   public int generateBytes(byte[] out, int outOff, int len) {
      if (out.length - len >= outOff) {
         long var4;
         long var10000 = var4 = (long)len;
         int var6 = this.digest.getDigestSize();
         if (var10000 <= 8589934591L) {
            int var10 = (int)((var4 + (var4 = (long)var6) - 1L) / var4);
            byte[] var5 = new byte[this.digest.getDigestSize()];
            int var7 = this.counterStart;

            for(int var8 = 0; var8 < var10; ++var8) {
               byte[] var10006 = this.shared;
               this.digest.update(var10006, 0, var10006.length);
               this.digest.update((byte)(var7 >> 24));
               this.digest.update((byte)(var7 >> 16));
               this.digest.update((byte)(var7 >> 8));
               this.digest.update((byte)var7);
               byte[] var9;
               if ((var9 = this.iv) != null) {
                  this.digest.update(var9, 0, var9.length);
               }

               this.digest.doFinal(var5, 0);
               if (len > var6) {
                  System.arraycopy(var5, 0, out, outOff, var6);
                  outOff += var6;
                  len -= var6;
               } else {
                  System.arraycopy(var5, 0, out, outOff, len);
               }

               ++var7;
            }

            this.digest.reset();
            return len;
         } else {
            throw new IllegalArgumentException("Output length too large");
         }
      } else {
         throw new DataLengthException("output buffer too small");
      }
   }
}
